@@ -1,11 +1,34 @@
-package App::ClusterSSH::Base;
-
 use warnings;
 use strict;
+
+package App::ClusterSSH::Base;
+$App::ClusterSSH::Base::VERSION = '4.16';
+# ABSTRACT: App::ClusterSSH::Base - Base object provding utility functions
+
+=head1 SYNOPSIS
+
+    use base qw/ App::ClusterSSH::Base /;
+
+    # in object new method
+    sub new {
+        ( $class, $arg_ref ) = @_;
+        my $self = $class->SUPER::new($arg_ref);
+        return $self;
+    }
+
+=head1 DESCRIPTION
+
+Base object to provide some utility functions on objects - should not be 
+used directly
+
+=cut
+
 use Carp;
 use App::ClusterSSH::L10N;
 
-use Exception::Class (
+use Module::Load;
+
+use Exception::Class 1.31 (
     'App::ClusterSSH::Exception',
     'App::ClusterSSH::Exception::Config' => {
         fields => 'unknown_config',
@@ -16,11 +39,7 @@ use Exception::Class (
     'App::ClusterSSH::Exception::Getopt',
 );
 
-# Don't use SVN revision as it can cause problems
-use version;
-our $VERSION = version->new('0.02');
-
-my $debug_level = 4;
+my $debug_level = $ENV{CLUSTERSSH_DEBUG} || 0;
 our $language = 'en';
 our $language_handle;
 our $app_configuration;
@@ -29,14 +48,13 @@ sub new {
     my ( $class, %args ) = @_;
 
     my $config = {
-        lang  => 'en',
-        debug => 0,
+        lang => 'en',
         %args,
     };
 
     my $self = bless $config, $class;
 
-    $self->set_debug_level( $config->{debug} );
+    $self->set_debug_level( $config->{debug} ) if ( $config->{debug} );
     $self->set_lang( $config->{lang} );
 
     $self->debug(
@@ -112,7 +130,7 @@ sub debug_level {
     return $debug_level;
 }
 
-sub output {
+sub stdout_output {
     my ( $self, @text ) = @_;
     print @text, $/;
     return $self;
@@ -121,7 +139,7 @@ sub output {
 sub debug {
     my ( $self, $level, @text ) = @_;
     if ( $level <= $debug_level ) {
-        $self->output(@text);
+        $self->stdout_output(@text);
     }
     return $self;
 }
@@ -143,9 +161,21 @@ sub config {
         );
     }
 
+    return $self->{parent}->{config}
+        if $self->{parent}
+        && ref $self->{parent} eq "HASH"
+        && $self->{parent}->{config};
+
     return $app_configuration;
 }
 
+sub options {
+    my ($self) = @_;
+    return $self->{parent}->{options}
+        if $self->{parent} && $self->{parent}->{options};
+    return;
+}
+
 sub set_config {
     my ( $self, $config ) = @_;
 
@@ -271,29 +301,33 @@ sub parent {
     return $self->{parent};
 }
 
-1;
+sub sort {
+    my $self = shift;
 
-=pod
+    my $sort = sub { sort @_ };
 
-=head1 NAME
+    return $sort
+        unless ref( $self->config() ) eq "HASH"
+        && $self->config()->{'use_natural_sort'};
 
-App::ClusterSSH::Base - Base object provding utility functions
-
-=head1 SYNOPSIS
-
-    use base qw/ App::ClusterSSH::Base /;
-
-    # in object new method
-    sub new {
-        ( $class, $arg_ref ) = @_;
-        my $self = $class->SUPER::new($arg_ref);
-        return $self;
+    # if the user has asked for natural sorting we need to include an extra
+    # module
+    if ( $self->config()->{'use_natural_sort'} ) {
+        eval { Module::Load::load('Sort::Naturally'); };
+        if ($@) {
+            warn(
+                "natural sorting requested but unable to load Sort::Naturally: $@\n"
+            );
+        }
+        else {
+            $sort = sub { Sort::Naturally::nsort(@_) };
+        }
     }
 
-=head1 DESCRIPTION
+    return $sort;
+}
 
-Base object to provide some utility functions on objects - should not be 
-used directly
+1;
 
 =head1 METHODS
 
@@ -335,15 +369,19 @@ Using the App::ClusterSSH/L10N/{lang}.pm module convert the  given text to
 appropriate language.  See L<App::ClusterSSH::L10N> for more details.  Essentially 
 a wrapper to maketext in Locale::Maketext
 
-=item $obj->output(@);
+=item $obj->stdout_output(@);
 
 Output text on STDOUT.
 
-=item $ovj->parent;
+=item $obj->parent;
 
-Reutrned the object that is the parent of this one, if it was set when the 
+Returned the object that is the parent of this one, if it was set when the 
 object was created
 
+=item %obj->options;
+
+Accessor to configured options, if it is set up by this point
+
 =item $obj->exit;
 
 Stub to allow program to exit neatly from wherever in the code
@@ -357,27 +395,14 @@ hasnt been called
 
 Set the config to the given value - croaks if has already been called
 
+=item $sort = $obj->sort
+
+Code reference used to sort lists; if configured (and installed) use
+Sort;:Naturally, else use perl sort
+
 =item %results = $obj->load_file( filename => '/path/to/file', type => '(cluster|config}' )
 
 Load in the specified file and return a hash, parsing the file depending on
 wther it is a config file (key = value) or cluster file (key value)
 
 =back
-
-=head1 AUTHOR
-
-Duncan Ferguson, C<< <duncan_j_ferguson at yahoo.co.uk> >>
-
-=head1 LICENSE AND COPYRIGHT
-
-Copyright 1999-2015 Duncan Ferguson.
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See http://dev.perl.org/licenses/ for more information.
-
-=cut
-
-1;
