@@ -1,10 +1,17 @@
-package App::ClusterSSH::Config;
-
 use strict;
 use warnings;
 
-use version;
-our $VERSION = version->new('0.02');
+package App::ClusterSSH::Config;
+$App::ClusterSSH::Config::VERSION = '4.16';
+# ABSTRACT: ClusterSSH::Config - Object representing application configuration
+
+=head1 SYNOPSIS
+
+=head1 DESCRIPTION
+
+Object representing application configuration
+
+=cut
 
 use Carp;
 use Try::Tiny;
@@ -42,6 +49,10 @@ my %default_config = (
     key_macros_enable       => "Alt-p",
     key_paste               => "Control-v",
     key_username            => "Alt-u",
+    key_user_1              => "Alt-1",
+    key_user_2              => "Alt-2",
+    key_user_3              => "Alt-3",
+    key_user_4              => "Alt-4",
     mouse_paste             => "Button-2",
     auto_quit               => "yes",
     auto_close              => 5,
@@ -77,6 +88,8 @@ my %default_config = (
     extra_tag_file           => '',
     extra_cluster_file       => '',
     external_cluster_command => '',
+    external_command_mode    => '0600',
+    external_command_pipe    => '',
 
     unmap_on_redraw => "no",    # Debian #329440
 
@@ -85,6 +98,9 @@ my %default_config = (
     history_height => 10,
 
     command             => q{},
+    command_pre         => q{},
+    command_post        => q{},
+    hide_menu           => 0,
     max_host_menu_items => 30,
 
     macros_enabled   => 'yes',
@@ -93,17 +109,34 @@ my %default_config = (
     macro_username   => '%u',
     macro_newline    => '%n',
     macro_version    => '%v',
+    macro_user_1     => '%1',
+    macro_user_2     => '%2',
+    macro_user_3     => '%3',
+    macro_user_4     => '%4',
+
+    macro_user_1_command => '',
+    macro_user_2_command => '',
+    macro_user_3_command => '',
+    macro_user_4_command => '',
 
     max_addhost_menu_cluster_items => 6,
     menu_send_autotearoff          => 0,
     menu_host_autotearoff          => 0,
 
+    unique_servers    => 0,
     use_all_a_records => 0,
 
     send_menu_xml_file => $ENV{HOME} . '/.clusterssh/send_menu',
 
+    auto_wm_decoration_offsets => "no",    # Debian #842965
+
     # don't set username here as takes precendence over ssh config
     user => '',
+    rows => -1,
+    cols => -1,
+
+    fillscreen => "no",
+
 );
 
 sub new {
@@ -261,6 +294,15 @@ sub parse_config_file {
         if ( $read_config{terminal_font} );
 
     $self->validate_args(%read_config);
+
+    # Look at the user macros and if not set remove the hotkey for them
+    for my $i (qw/ 1 2 3 4 /) {
+        if ( !$self->{"macro_user_${i}_command"} ) {
+            delete $self->{"key_user_${i}"};
+        }
+    }
+
+    return $self;
 }
 
 sub load_configs {
@@ -489,7 +531,7 @@ sub dump {
     my ( $self, $no_exit, ) = @_;
 
     $self->debug( 3, 'Dumping config to STDOUT' );
-    print( '# Configuration dump produced by "cssh -u"', $/ );
+    print( '# Configuration dump produced by "cssh -d"', $/ );
 
     foreach my $key ( sort keys %$self ) {
         my $comment = '';
@@ -515,18 +557,6 @@ sub dump {
 
 1;
 
-=pod
-
-=head1 NAME
-
-ClusterSSH::Config - Object representing application configuration
-
-=head1 SYNOPSIS
-
-=head1 DESCRIPTION
-
-Object representing application configuration
-
 =head1 METHODS
 
 =over 4
@@ -567,21 +597,3 @@ are loaded).
 Write currently defined configuration to STDOUT
 
 =back
-
-=head1 AUTHOR
-
-Duncan Ferguson, C<< <duncan_j_ferguson at yahoo.co.uk> >>
-
-=head1 LICENSE AND COPYRIGHT
-
-Copyright 1999-2015 Duncan Ferguson.
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See http://dev.perl.org/licenses/ for more information.
-
-=cut
-
-1;
