@@ -1,10 +1,25 @@
-package App::ClusterSSH::Host;
-
 use strict;
 use warnings;
 
-use version;
-our $VERSION = version->new('0.03');
+package App::ClusterSSH::Host;
+$App::ClusterSSH::Host::VERSION = '4.16';
+# ABSTRACT: ClusterSSH::Host - Object representing a host.
+
+=head1 SYNOPSIS
+
+    use ClusterSSH::Host;
+
+    my $host = ClusterSSH::Host->new({
+        hostname => 'hostname',
+    });
+    my $host = ClusterSSH::Host->parse_host_string('username@hostname:1234');
+
+=head1 DESCRIPTION
+
+Object representing a host.  Include details to contact the host such as
+hostname/ipaddress, username and port.
+
+=cut
 
 use Carp;
 use Net::hostent;
@@ -37,30 +52,44 @@ sub new {
 
     # load in ssh hostname for later use
     if ( !%ssh_hostname_for || !$ssh_configs_read{ $self->{ssh_config} } ) {
-        $ssh_configs_read{ $self->{ssh_config} } = 1;
-        if ( open( my $ssh_config_fh, '<', $self->{ssh_config} ) ) {
-            while ( my $line = <$ssh_config_fh> ) {
-                chomp $line;
-                next unless ( $line =~ m/^\s*host\s+(.*)/i );
+        $self->read_ssh_file( $self->{ssh_config} );
 
-                # account for multiple declarations of hosts
-                $ssh_hostname_for{$_} = 1 foreach ( split( /\s+/, $1 ) );
-            }
-            close($ssh_config_fh);
-
-            $self->debug( 5, 'Have the following ssh hostnames' );
-            $self->debug( 5, '  "', $_, '"' )
-                foreach ( sort keys %ssh_hostname_for );
-        }
-        else {
-            $self->debug( 3, 'Unable to read ',
-                $self->{ssh_config}, ': ', $!, $/ );
-        }
+        $self->debug( 5, 'Have the following ssh hostnames' );
+        $self->debug( 5, '  "', $_, '"' )
+            foreach ( sort keys %ssh_hostname_for );
     }
 
     return $self;
 }
 
+sub read_ssh_file($$) {
+    my ($self)     = shift;
+    my ($filename) = glob(shift);
+    $self->debug( 3, 'Reading SSH file: ', $filename );
+
+    $ssh_configs_read{$filename} = 1;
+
+    if ( open( my $ssh_config_fh, '<', $filename ) ) {
+        while ( my $line = <$ssh_config_fh> ) {
+            chomp $line;
+
+            if ( $line =~ /^\s*include\s+(.+)/i ) {
+                $self->read_ssh_file($1);
+                next;
+            }
+
+            next unless ( $line =~ m/^\s*host\s+(.*)/i );
+
+            # account for multiple declarations of hosts
+            $ssh_hostname_for{$_} = 1 foreach ( split( /\s+/, $1 ) );
+        }
+        close($ssh_config_fh);
+    }
+    else {
+        $self->debug( 3, 'Unable to read ', $filename, ': ', $!, $/ );
+    }
+}
+
 sub get_hostname {
     my ($self) = @_;
     return $self->{hostname};
@@ -295,9 +324,11 @@ sub check_ssh_hostname {
         $self->get_hostname );
 
     if ( $ssh_hostname_for{ $self->get_hostname } ) {
+        $self->debug( 5, 'Found' );
         return 1;
     }
     else {
+        $self->debug( 5, 'Not found' );
         return 0;
     }
 }
@@ -312,26 +343,6 @@ use overload (
 
 1;
 
-=pod
-
-=head1 NAME
-
-ClusterSSH::Host - Object representing a host.
-
-=head1 SYNOPSIS
-
-    use ClusterSSH::Host;
-
-    my $host = ClusterSSH::Host->new({
-        hostname => 'hostname',
-    });
-    my $host = ClusterSSH::Host->parse_host_string('username@hostname:1234');
-
-=head1 DESCRIPTION
-
-Object representing a host.  Include details to contact the host such as
-hostname/ipaddress, username and port.
-
 =head1 METHODS
 
 =over 4
@@ -386,6 +397,11 @@ Given a host string, returns a host object.  Parses hosts such as
 Check the objects hostname to see whether or not it may be configured within 
 the users F< $HOME/.ssh/config > configuration file
 
+=item read_ssh_file
+
+Method to ease reading in ssh configuration files.  Used for grabbing
+hostnames for validation when used in clusters
+
 =over 4
 
 =item host
@@ -410,21 +426,3 @@ the IPv6 address or a port definition?) and assumes it is part of address.
 Use brackets to avoid seeing warning.
 
 =back
-
-=head1 AUTHOR
-
-Duncan Ferguson, C<< <duncan_j_ferguson at yahoo.co.uk> >>
-
-=head1 LICENSE AND COPYRIGHT
-
-Copyright 1999-2016 Duncan Ferguson.
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See http://dev.perl.org/licenses/ for more information.
-
-=cut
-
-1;
